export const deleteValue = Symbol('deleteValue');
function deepAssign(currentObj, newObj) {
    if (!newObj)
        return currentObj;
    for (const [key, value] of Object.entries(newObj)) {
        if (!currentObj[key]) {
            continue;
        }
        if (typeof value == 'object' && value && value.constructor.name === 'Object') {
            newObj[key] = deepAssign(currentObj[key], newObj[key]);
        }
    }
    return { ...currentObj, ...newObj };
}
export function applyMutation(conditions, obj, option = { top: true }) {
    const defaultOption = { keepMutation: false };
    Object.assign(defaultOption, option);
    let result = { ...obj };
    let lastObjResult = result;
    if (result.mutate) {
        for (const mutateKey of Object.keys(result.mutate)) {
            const matchCondition = conditions.find((c) => (typeof c == 'string' ? c : c.condition) == mutateKey);
            if (matchCondition) {
                const mutation = result.mutate[mutateKey];
                if (mutation === deleteValue) {
                    result = deleteValue;
                }
                else if (typeof mutation == 'function') {
                    let args;
                    if (typeof matchCondition != 'string') {
                        args = matchCondition.args();
                    }
                    result = mutation.bind(this)(result, ...(Array.isArray(args) ? args : [args]));
                }
                else {
                    result = deepAssign(lastObjResult, mutation);
                }
                if (result && typeof result == 'object')
                    lastObjResult = result;
            }
        }
    }
    for (const [key, value] of Object.entries(result)) {
        if (value && typeof value == 'object' && value.constructor.name === 'Object') {
            result[key] = applyMutation(conditions, value, { ...option, top: false });
        }
        if (result[key] == deleteValue)
            delete result[key];
    }
    if (!option.keepMutation) {
        if (typeof result == 'object' && 'mutate' in result)
            delete result['mutate'];
    }
    if (option.top && result == deleteValue)
        result = undefined;
    return result;
}
